
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <ctype.h>
#include <unixio.h>

#define VERSION "0.1 alpha, 31-Mar-2003"

int verbose = 0;
char *progname;

void help()
{
	printf("TRD truncate program, version "\
		VERSION\
		"\n(C) Kirill Frolov 2003\n"\
		"usage: %s [-h] [-v] filename.trd [filename...]\n"\
		"options:\n"\
		"-v	be more verbose\n"\
		"-h	show this help\n\n",
			progname);
	exit(0);
}


int trdtrunc(char *file)
{
int x, f;
unsigned char buf[0x20];
int blocks;
char name[12];

	if (f=open(file, 2), f<0) {
		fprintf(stderr, "%s: %s: %s\n",
			progname, file, strerror(errno));
		exit(1);
	}
	lseek(f, 0x8e0, 0);
	read(f, buf, 0x20);
	
	if (buf[7]!=0x10) {
		fprintf(stderr, "%s: not tr-dos disk image!\n", file);
		return -1;
	}

	blocks=(buf[1]+16*buf[2]);
	if ( (buf[3]<0x16) || (buf[3]>0x19)
		|| (buf[4]>128) || (buf[0x14]>128)
		|| ((buf[5]+buf[6]*256+blocks)>=4096)
	 ) {
		fprintf(stderr, "%s: incorrect disk information record!\n", file);
		return -1;
	 }
	
	lseek(f, (long)blocks*256, 0);
	write(f, NULL, 0);	/* truncate under M$-DOS */
	close(f);

	if (verbose) {
		for (x=0; x<8; x++) {
			if (isprint(buf[0x15+x])) name[x]=buf[0x15+x];
				else name[x]='?';
		}
		name[8]=0;
		printf("%s: \"%s\"  %d(%d) files, %d sectors, trunc. to %ld bytes\n",
			file, name, buf[4], buf[0x14]+buf[4], blocks, (long)blocks*256);
	}
	
	return 0;
}


int main(int argc, char *argv[])
{
int x;
	progname=strrchr(argv[0], '\\')+1;
	if ( (progname==NULL) || (progname[0]==0)) progname=argv[0];
	if (argc<=1) {
		fprintf(stderr, "%s: no arguments\n"\
			"(try %s -h)\n", progname, progname);
		exit(2);
	}

	for (x=1; x<argc; x++) {
		if (argv[x][0]=='-') {
			switch (argv[x][1]) {
				case '?':
				case 'h':
				case 'H': help();
					break;
				case 'v':
				case 'V': verbose=1;
					break;
				default:
					fprintf(stderr, "%s: unknown option: %s\n",
						progname, argv[x]);
					exit(2);
			}
			argv[x]=NULL;
		}
	}
			
	
	for (x=1; x<argc; x++) {
		if (argv[x]!=NULL) trdtrunc(argv[x]);
	}
	
	return 0;
}

